@@ -3,7 +3,7 @@ package FSA::Rules;
 use strict;
 use 5.006_002;
 use Scalar::Util 1.01 ();
-$FSA::Rules::VERSION = '0.32';
+$FSA::Rules::VERSION = '0.33';
 
 =head1 Name
 
@@ -381,6 +381,9 @@ sub new {
                     exec    => $exec,
                     message => $message,
                 };
+
+                # Weaken the circular reference.
+                Scalar::Util::weaken $rules[-1]->{state};
             }
             $def->{rules} = \@rules;
         } else {
@@ -959,6 +962,7 @@ sub stacktrace {
     local $Data::Dumper::Indent    = 1;
     local $Data::Dumper::Quotekeys = 0;
     local $Data::Dumper::Sortkeys  = 1;
+    local  $Data::Dumper::Useperl  = $] < 5.008;
     foreach my $state (@$states) {
         $stacktrace .= "State: $state->[0]\n";
         $stacktrace .= Data::Dumper::Dumper($state->[1]);
